from enum import IntEnum


class DiplomacyState(IntEnum):
    ALLY = 0
    NEUTRAL = 1
    ENEMY = 3


class Operator(IntEnum):
    SET = 1
    ADD = 2
    SUBTRACT = 3
    MULTIPLY = 4
    DIVIDE = 5


class ButtonLocation(IntEnum):
    LOCATION_0_0 = 0
    LOCATION_1_0 = 2
    LOCATION_2_0 = 3
    LOCATION_3_0 = 4
    LOCATION_4_0 = 5
    LOCATION_0_1 = 6
    LOCATION_1_1 = 7
    LOCATION_2_1 = 8
    LOCATION_3_1 = 9
    LOCATION_4_1 = 10
    LOCATION_0_2 = 11
    LOCATION_1_2 = 12
    LOCATION_2_2 = 13
    LOCATION_3_2 = 14
    # LOCATION_4_2 = 15? (Actually placed at 0_0 at the moment)


class PanelLocation(IntEnum):
    TOP = 0
    """Panel at the top of the screen. ~13% from the top"""
    BETWEEN = 1
    """Panel between the top and the center of the screen. ~33% from the top"""
    CENTER = 2
    """Panel close to the center of the screen. ~45% from the top"""


class TimeUnit(IntEnum):
    YEARS = 2
    """In-Game years. A year is 5 seconds in-game time."""
    MINUTES = 1
    """In-Game minutes."""
    SECONDS = 0
    """In-Game seconds."""


class VisibilityState(IntEnum):
    VISIBLE = 0
    EXPLORED = 1
    INVISIBLE = 2


class DifficultyLevel(IntEnum):
    EASIEST = 4
    STANDARD = 3
    MODERATE = 2
    HARD = 1
    HARDEST = 0
    # EXTREME = 5  # ???


class TechnologyState(IntEnum):
    DISABLED = 4
    NOT_READY = 3
    READY = 2
    RESEARCHING = 1
    DONE = 0
    # QUEUED = 5  # ???


class Comparison(IntEnum):
    EQUAL = 0
    LESS = 1
    LARGER = 2
    LESS_OR_EQUAL = 3
    LARGER_OR_EQUAL = 4


class ObjectAttribute(IntEnum):
    HIT_POINTS = 0
    LINE_OF_SIGHT = 1
    GARRISON_CAPACITY = 2
    UNIT_SIZE_X = 3
    UNIT_SIZE_Y = 4
    MOVEMENT_SPEED = 5
    ROTATION_SPEED = 6
    ARMOR = 8
    ATTACK = 9
    ATTACK_RELOAD_TIME = 10
    ACCURACY_PERCENT = 11
    MAX_RANGE = 12
    WORK_RATE = 13
    CARRY_CAPACITY = 14
    BASE_ARMOR = 15
    PROJECTILE_UNIT = 16
    ICON_GRAPHICS_ANGLE = 17
    TERRAIN_DEFENSE_BONUS = 18
    ENABLE_SMART_PROJECTILES = 19
    MINIMUM_RANGE = 20
    ARNOUNT_OF_1ST_RESOURCES = 21
    BLAST_WIDTH = 22
    SEARCH_RADIUS = 23
    HERO_STATUS = 40
    FRAME_DELAY = 41
    TRAIN_LOCATION = 42
    TRAIN_BUTTON = 43
    BLAST_LEVEL = 44
    OBJECT_NAME_ID = 50
    SHORT_DESCRIPTION_ID = 51
    TERRAIN_RESTRICTION_ID = 53
    DEAD_UNIT_ID = 57
    RESOURCE_COSTS = 100
    TRAIN_TIME = 101
    TOTAL_MISSILES = 102
    FOOD_COSTS = 103
    WOOD_COSTS = 104
    GOLD_COSTS = 105
    STONE_COSTS = 106
    MAX_TOTAL_MISSILES = 107
    REGENERATION_RATE = 109


class Attribute(IntEnum):
    FOOD_STORAGE = 0
    WOOD_STORAGE = 1
    STONE_STORAGE = 2
    GOLD_STORAGE = 3
    POPULATION_HEADROOM = 4
    CONVERSION_RANGE = 5
    CURRENT_AGE = 6
    RELICS_CAPTURED = 7
    UNUSED_RESOURCE_008 = 8
    TRADE_GOODS = 9
    UNUSED_RESOURCE_010 = 10
    CURRENT_POPULATION = 11
    CORPSE_DECAY_TIME = 12
    REMARKABLE_DISCOVERY = 13
    MONUMENTS_CAPTURED = 14
    MEAT_STORAGE = 15
    BERRY_STORAGE = 16
    FISH_STORAGE = 17
    UNUSED_RESOURCE_018 = 18
    TOTAL_UNITS_OWNED = 19
    UNITS_KILLED = 20
    TECHNOLOGY_COUNT = 21
    PERCENT_MAP_EXPLORED = 22
    CASTLE_AGE_TECH_ID = 23
    IMPERIAL_AGE_TECH_ID = 24
    FEUDAL_AGE_TECH_ID = 25
    ATTACK_WARNING_SOUND_ID = 26
    ENABLE_MONK_CONVERSION = 27
    ENABLE_BUILDING_CONVERSION = 28
    UNUSED_RESOURCE_029 = 29
    UNUSED_RESOURCE_030 = 30
    UNUSED_RESOURCE_031 = 31
    BONUS_POPULATION_CAP = 32
    FOOD_MAINTENANCE = 33
    FAITH = 34
    FAITH_RECHARGING_RATE = 35
    FARM_FOOD_AMOUNT = 36
    CIVILIAN_POPULATION = 37
    UNUSED_RESOURCE_038 = 38
    ALL_TECHS_ACHIEVED = 39
    MILITARY_POPULATION = 40
    CONVERSIONS = 41
    STANDING_NDERS = 42
    RAZINGS = 43
    KILL_RATIO = 44
    SURVIVAL_TO_FINISH = 45
    TRIBUTE_INEFFICIENCY = 46
    GOLD_MINING_PRODUCTIVITY = 47
    TOWN_CENTER_UNAVAILABLE = 48
    GOLD_COUNTER = 49
    REVEAL_ALLY = 50
    UNUSED_RESOURCE_051 = 51
    MONASTERIES = 52
    TRIBUTE_SENT = 53
    ALL_MONUMENTS_CAPTURED = 54
    ALL_RELICS_CAPTURED = 55
    ORE_STORAGE = 56
    KIDNAP_STORAGE = 57
    DARK_AGE_TECH_ID = 58
    UNUSED_RESOURCE_059 = 59
    UNUSED_RESOURCE_060 = 60
    UNUSED_RESOURCE_061 = 61
    BUILDING_HOUSING_RATE = 62
    TAX_GATHER_RATE = 63
    GATHER_ACCUMULATOR = 64
    SALVAGE_DECAY_RATE = 65
    UNUSED_RESOURCE_066 = 66
    CAN_CONVERT = 67
    HIT_POINTS_KILLED = 68
    KILLED_P1 = 69
    KILLED_P2 = 70
    KILLED_P3 = 71
    KILLED_P4 = 72
    KILLED_P5 = 73
    KILLED_P6 = 74
    KILLED_P7 = 75
    KILLED_P8 = 76
    CONVERSION_RESISTANCE = 77
    TRADE_VIG_RATE = 78
    STONE_MINING_PRODUCTIVITY = 79
    QUEUED_UNITS = 80
    TRAINING_COUNT = 81
    START_WITH_UNIT_444_PTWC = 82
    BOARDING_RECHARGE_RATE = 83
    STARTING_VILLAGERS = 84
    RESEARCH_COST_MODIFIER = 85
    RESEARCH_TIME_MODIFIER = 86
    CONVERT_BOATS = 87
    FISH_TRAP_FOOD_AMOUNT = 88
    HEAL_RATE_MODIFIER = 89
    HEALING_RANGE = 90
    STARTING_FOOD = 91
    STARTING_WOOD = 92
    STARTING_STONE = 93
    STARTING_GOLD = 94
    ENABLE_PTWC_KIDNAP_LOOT = 95
    BERSERKER_HEAL_TIMER = 96
    DOMINANT_SHEEP_CONTROL = 97
    BUILDING_COST_SUM = 98
    TECH_COST_SUM = 99
    RELIC_INCOME_SUM = 100
    TRADE_INCOME_SUM = 101
    P1_TRIBUTE = 102
    P2_TRIBUTE = 103
    P3_TRIBUTE = 104
    P4_TRIBUTE = 105
    P5_TRIBUTE = 106
    P6_TRIBUTE = 107
    P7_TRIBUTE = 108
    P8_TRIBUTE = 109
    P1_KILL_VALUE = 110
    P2_KILL_VALUE = 111
    P3_KILL_VALUE = 112
    P4_KILL_VALUE = 113
    P5_KILL_VALUE = 114
    P6_KILL_VALUE = 115
    P7_KILL_VALUE = 116
    P8_KILL_VALUE = 117
    P1_RAZINGS = 118
    P2_RAZINGS = 119
    P3_RAZINGS = 120
    P4_RAZINGS = 121
    P5_RAZINGS = 122
    P6_RAZINGS = 123
    P7_RAZINGS = 124
    P8_RAZINGS = 125
    P1_RAZING_VALUE = 126
    P2_RAZING_VALUE = 127
    P3_RAZING_VALUE = 128
    P4_RAZING_VALUE = 129
    P5_RAZING_VALUE = 130
    P6_RAZING_VALUE = 131
    P7_RAZING_VALUE = 132
    P8_RAZING_VALUE = 133
    STANDING_CASTLES = 134
    HIT_POINTS_RAZED = 135
    KILLS_BY_P1 = 136
    KILLS_BY_P2 = 137
    KILLS_BY_P3 = 138
    KILLS_BY_P4 = 139
    KILLS_BY_P5 = 140
    KILLS_BY_P6 = 141
    KILLS_BY_P7 = 142
    KILLS_BY_P8 = 143
    RAZINGS_BY_P1 = 144
    RAZINGS_BY_P2 = 145
    RAZINGS_BY_P3 = 146
    RAZINGS_BY_P4 = 147
    RAZINGS_BY_P5 = 148
    RAZINGS_BY_P6 = 149
    RAZINGS_BY_P7 = 150
    RAZINGS_BY_P8 = 151
    VALUE_KILLED_BY_OTHERS = 152
    VALUE_RAZED_BY_OTHERS = 153
    KILLED_BY_OTHERS = 154
    RAZED_BY_OTHERS = 155
    TRIBUTE_FROM_P1 = 156
    TRIBUTE_FROM_P2 = 157
    TRIBUTE_FROM_P3 = 158
    TRIBUTE_FROM_P4 = 159
    TRIBUTE_FROM_P5 = 160
    TRIBUTE_FROM_P6 = 161
    TRIBUTE_FROM_P7 = 162
    TRIBUTE_FROM_P8 = 163
    VALUE_CURENT_UNITS = 164
    VALUE_CURRENT_BUILDINGS = 165
    FOOD_TOTAL = 166
    WOOD_TOTAL = 167
    STONE_TOTAL = 168
    GOLD_TOTAL = 169
    TOTAL_VALUE_OF_KILLS = 170
    TOTAL_TRIBUTE_RECEIVED = 171
    TOTAL_VALUE_OF_RAZINGS = 172
    TOTAL_CASTLES_BUILT = 173
    TOTAL_WONDERS_BUILT = 174
    TRIBUTE_SCORE = 175
    CONVERT_MIN_ADJUSFIENT = 176
    CONVERT_MAX_ADJUSTMENT = 177
    CONVERT_RESIST_MIN_ADJUSTMENT = 178
    CONVERT_RESIST_MAX_ADJUSTMENT = 179
    CONVERT_BUILDING_MIN = 180
    CONVERT_BUILDING_MAX = 181
    CONVERT_BUILDING_CHANCE = 182
    REVEAL_ENEMY = 183
    VALUE_WONDERS_CASTLES = 184
    FOOD_SCORE = 185
    WOOD_SCORE = 186
    STONE_SCORE = 187
    GOLD_SCORE = 188
    CHOPPING_PRODUCTIVITY = 189
    FOOD_GATHERING_PRODUCTIVITY = 190
    RELIC_GOLD_PRODUCTION_RATE = 191
    CONVERTED_UNITS_DIE = 192
    THEOCRACY = 193
    CRENELLATIONS = 194
    CONSTRUCTION_RATE_MODIFIER = 195
    HUN_WONDER_DISCOUNT = 196
    SPIES_DISCOUNT = 197
    UNUSED_RESOURCE_198 = 198
    UNUSED_RESOURCE_199 = 199
    UNUSED_RESOURCE_200 = 200
    UNUSED_RESOURCE_201 = 201
    UNUSED_RESOURCE_202 = 202
    UNUSED_RESOURCE_203 = 203
    UNUSED_RESOURCE_204 = 204
    FEITORIA_FOOD_PRODUCTIVITY = 205
    FEITORIA_WOOD_PRODUCTIVITY = 206
    FEITORIA_STONE_PRODUCTIVITY = 207
    FEITORIA_GOLD_PRODUCTIVITY = 208
    REVEAL_ENEMY_TOWN_CENTERS = 209
    RELICS_VISIBLE_ON_MAP = 210
    ELEVATION_HIGHER_BONUS = 211
    ELEVATION_LOWER_BONUS = 212
    RAIDING_PRODUCTIVITY = 213
    MERCENARY_KIPCHAK_COUNT = 214
    MERCENARY_KIPCHAK_LIMIT = 215
    SHEPHERD_PRODUCTIVITY = 216
    SHARED_LINE_OF_SIGHT = 217



from AoE2ScenarioParser.helper.datatype import DataType
from AoE2ScenarioParser.helper.retriever import Retriever
from AoE2ScenarioParser.pieces.structs.aoe2_struct import AoE2Struct


class PlayerDataFourStruct(AoE2Struct):
    def __init__(self, parser_obj=None, data=None):
        retrievers = [
            Retriever("Food, duplicate", DataType("f32")),
            Retriever("Wood, duplicate", DataType("f32")),
            Retriever("Gold, duplicate", DataType("f32")),
            Retriever("Stone, duplicate", DataType("f32")),
            Retriever("'Ore X', duplicate", DataType("f32")),
            Retriever("Trade Goods, duplicate", DataType("f32")),
            Retriever("Population limit", DataType("f32")),
        ]

        super().__init__("Player Data #4", retrievers, parser_obj, data)



from AoE2ScenarioParser.helper.datatype import DataType
from AoE2ScenarioParser.helper.retriever import Retriever
from AoE2ScenarioParser.pieces.structs.aoe2_struct import AoE2Struct
from AoE2ScenarioParser.pieces.structs.condition import ConditionStruct
from AoE2ScenarioParser.pieces.structs.effect import EffectStruct


class TriggerStruct(AoE2Struct):
    def __init__(self, parser_obj=None, data=None):
        retrievers = [
            Retriever("Enabled", DataType("u32")),
            Retriever("Looping", DataType("s8")),
            Retriever("Description string Table ID", DataType("s32")),
            Retriever("Act as objective", DataType("u8")),
            Retriever("Description order (in objectives)", DataType("u32")),
            Retriever("Make header", DataType("u8")),
            Retriever("Short description string Table ID", DataType("s32")),
            Retriever("Display on screen", DataType("u8")),
            Retriever("Unknown", DataType("5")),
            Retriever("Mute objectives", DataType("u8")),
            # Retriever("Trigger starting time", DataType("u32")),
            Retriever("Trigger description", DataType("str32")),
            Retriever("Trigger name", DataType("str32")),  # (max 44 characters in UI)
            Retriever("Short description", DataType("str32")),
            Retriever("Number of effects", DataType("s32"), save_as="number_of_effects"),
            Retriever("Effect data", DataType(EffectStruct), set_repeat="{number_of_effects}"),
            Retriever("Effect display order array", DataType("s32"), set_repeat="{number_of_effects}"),
            Retriever("Number of conditions", DataType("s32"), save_as="number_of_conditions"),
            Retriever("Condition data", DataType(ConditionStruct), set_repeat="{number_of_conditions}"),
            Retriever("Condition display order array", DataType("s32"), set_repeat="{number_of_conditions}"),
        ]

        super().__init__("Trigger", retrievers, parser_obj, data)



from enum import IntEnum

from bidict import bidict


class Effect(IntEnum):
    NONE = 0
    """Attributes for the **none** effect are: \n
    ... none... Just like Conditions... People these days... """
    CHANGE_DIPLOMACY = 1
    """Attributes for the **change_diplomacy** effect are: \n
    - diplomacy
    - player_source
    - player_target"""
    RESEARCH_TECHNOLOGY = 2
    """Attributes for the **research_technology** effect are: \n
    - player_source
    - technology
    - force_research_technology"""
    SEND_CHAT = 3
    """Attributes for the **send_chat** effect are: \n
    - player_source
    - string_id
    - message
    - sound_name"""
    PLAY_SOUND = 4
    """Attributes for the **play_sound** effect are: \n
    - player_source
    - location_x
    - location_y
    - sound_name"""
    TRIBUTE = 5
    """Attributes for the **tribute** effect are: \n
    - quantity
    - tribute_list
    - player_source
    - player_target"""
    UNLOCK_GATE = 6
    """Attributes for the **unlock_gate** effect are: \n
    - number_of_units_selected
    - selected_object_id"""
    LOCK_GATE = 7
    """Attributes for the **lock_gate** effect are: \n
    - number_of_units_selected
    - selected_object_id"""
    ACTIVATE_TRIGGER = 8
    """Attributes for the **activate_trigger** effect are: \n
    - trigger_id"""
    DEACTIVATE_TRIGGER = 9
    """Attributes for the **deactivate_trigger** effect are: \n
    - trigger_id"""
    AI_SCRIPT_GOAL = 10
    """Attributes for the **deactivate_trigger** effect are: \n
    - ai_script_goal"""
    CREATE_OBJECT = 11
    """Attributes for the **create_object** effect are: \n
    - object_list_unit_id
    - player_source
    - location_x
    - location_y
    - item_id
    - facet"""
    TASK_OBJECT = 12
    """Attributes for the **task_object** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - location_x
    - location_y
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    DECLARE_VICTORY = 13
    """Attributes for the **declare_victory** effect are: \n
    - player_source
    - enabled_or_victory"""
    KILL_OBJECT = 14
    """Attributes for the **kill_object** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    REMOVE_OBJECT = 15
    """Attributes for the **remove_object** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    CHANGE_VIEW = 16
    """Attributes for the **change_view** effect are: \n
    - player_source
    - location_x
    - location_y
    - scroll"""
    UNLOAD = 17
    """Attributes for the **unload** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - location_x
    - location_y
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    CHANGE_OWNERSHIP = 18
    """Attributes for the **change_ownership** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - player_target
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - flash_object
    - selected_object_id"""
    PATROL = 19
    """Attributes for the **patrol** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - location_x
    - location_y
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    DISPLAY_INSTRUCTIONS = 20
    """Attributes for the **display_instructions** effect are: \n
    - object_list_unit_id
    - player_source
    - string_id
    - display_time
    - instruction_panel_position
    - play_sound
    - message
    - sound_name"""
    CLEAR_INSTRUCTIONS = 21
    """Attributes for the **clear_instructions** effect are: \n
    - instruction_panel_position"""
    FREEZE_OBJECT = 22
    """Attributes for the **freeze_object** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    USE_ADVANCED_BUTTONS = 23
    """Attributes for the **use_advanced_buttons** effect are: \n
    None. \n
    Please don't use this effect. Please."""
    DAMAGE_OBJECT = 24
    """Attributes for the **damage_object** effect are: \n
    - quantity
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    PLACE_FOUNDATION = 25
    """Attributes for the **place_foundation** effect are: \n
    - object_list_unit_id
    - player_source
    - location_x
    - location_y"""
    CHANGE_OBJECT_NAME = 26
    """Attributes for the **change_object_name** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - string_id
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - message
    - selected_object_id"""
    CHANGE_OBJECT_HP = 27
    """Attributes for the **change_object_hp** effect are: \n
    - quantity
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - operation
    - selected_object_id"""
    CHANGE_OBJECT_ATTACK = 28
    """Attributes for the **change_object_attack** effect are: \n
    - aa_quantity
    - aa_armor_or_attack_type
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - operation
    - selected_object_id"""
    STOP_OBJECT = 29
    """Attributes for the **stop_object** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    ATTACK_MOVE = 30
    """Attributes for the **attack_move** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - location_x
    - location_y
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    CHANGE_OBJECT_ARMOR = 31
    """Attributes for the **change_object_armor** effect are: \n
    - aa_quantity
    - aa_armor_or_attack_type
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - operation
    - selected_object_id"""
    CHANGE_OBJECT_RANGE = 32
    """Attributes for the **change_object_range** effect are: \n
    - quantity
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - operation
    - selected_object_id"""
    CHANGE_OBJECT_SPEED = 33
    """Attributes for the **change_object_speed** effect are: \n
    - quantity
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    HEAL_OBJECT = 34
    """Attributes for the **heal_object** effect are: \n
    - quantity
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    TELEPORT_OBJECT = 35
    """Attributes for the **teleport_object** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - location_x
    - location_y
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - selected_object_id"""
    CHANGE_OBJECT_STANCE = 36
    """Attributes for the **change_object_stance** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - attack_stance
    - selected_object_id"""
    DISPLAY_TIMER = 37
    """Attributes for the **display_timer** effect are: \n
    - string_id
    - display_time
    - time_unit
    - variable_or_timer
    - message"""
    ENABLE_DISABLE_OBJECT = 38
    """Attributes for the **enable_disable_object** effect are: \n
    - object_list_unit_id
    - player_source
    - enabled_or_victory
    - item_id"""
    ENABLE_DISABLE_TECHNOLOGY = 39
    """Attributes for the **enable_disable_technology** effect are: \n
    - player_source
    - technology
    - enabled_or_victory
    - item_id"""
    CHANGE_OBJECT_COST = 40
    """Attributes for the **change_object_cost** effect are: \n
    - object_list_unit_id
    - player_source
    - food
    - wood
    - stone
    - gold"""
    SET_PLAYER_VISIBILITY = 41
    """Attributes for the **set_player_visibility** effect are: \n
    - player_source
    - player_target
    - visibility_state"""
    CHANGE_OBJECT_ICON = 42
    """Attributes for the **change_object_icon** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - object_list_unit_id_2
    - selected_object_id"""
    REPLACE_OBJECT = 43
    """Attributes for the **replace_object** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - player_target
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_group
    - object_type
    - object_list_unit_id_2
    - selected_object_id"""
    CHANGE_OBJECT_DESCRIPTION = 44
    """Attributes for the **change_object_description** effect are: \n
    - object_list_unit_id
    - player_source
    - string_id
    - message"""
    CHANGE_PLAYER_NAME = 45
    """Attributes for the **change_player_name** effect are: \n
    - player_source
    - string_id
    - message"""
    CHANGE_TRAIN_LOCATION = 46
    """Attributes for the **change_train_location** effect are: \n
    - object_list_unit_id
    - player_source
    - object_list_unit_id_2
    - button_location"""
    CHANGE_RESEARCH_LOCATION = 47
    """Attributes for the **change_research_location** effect are: \n
    - player_source
    - technology
    - object_list_unit_id_2
    - button_location"""
    CHANGE_CIVILIZATION_NAME = 48
    """Attributes for the **change_civilization_name** effect are: \n
    - player_source
    - string_id
    - message"""
    CREATE_GARRISONED_OBJECT = 49
    """Attributes for the **create_garrisoned_object** effect are: \n
    - number_of_units_selected
    - object_list_unit_id
    - player_source
    - area_1_x
    - area_1_y
    - area_2_x
    - area_2_y
    - object_list_unit_id_2
    - selected_object_id"""
    ACKNOWLEDGE_AI_SIGNAL = 50
    """Attributes for the **acknowledge_ai_signal** effect are: \n
    - ai_signal_value"""
    MODIFY_ATTRIBUTE = 51
    """Attributes for the **modify_attribute** effect are: \n
    - quantity
    - object_list_unit_id
    - player_source
    - item_id
    - operation
    - object_attributes"""
    MODIFY_RESOURCE = 52
    """Attributes for the **modify_resource** effect are: \n
    - quantity
    - tribute_list
    - player_source
    - item_id
    - operation"""
    MODIFY_RESOURCE_BY_VARIABLE = 53
    """Attributes for the **modify_resource_by_variable** effect are: \n
    - tribute_list
    - player_source
    - item_id
    - operation
    - from_variable"""
    CHANGE_VARIABLE = 56
    """Attributes for the **change_variable** effect are: \n
    - quantity
    - operation
    - from_variable
    - message"""
    CLEAR_TIMER = 57
    """Attributes for the **clear_timer** effect are: \n
    - variable_or_timer"""


effect_names = bidict({
    0: "none",
    1: "change_diplomacy",
    2: "research_technology",
    3: "send_chat",
    4: "play_sound",
    5: "tribute",
    6: "unlock_gate",
    7: "lock_gate",
    8: "activate_trigger",
    9: "deactivate_trigger",
    10: "ai_script_goal",
    11: "create_object",
    12: "task_object",
    13: "declare_victory",
    14: "kill_object",
    15: "remove_object",
    16: "change_view",
    17: "unload",
    18: "change_ownership",
    19: "patrol",
    20: "display_instructions",
    21: "clear_instructions",
    22: "freeze_object",
    23: "use_advanced_buttons",
    24: "damage_object",
    25: "place_foundation",
    26: "change_object_name",
    27: "change_object_hp",
    28: "change_object_attack",
    29: "stop_object",
    30: "attack_move",
    31: "change_object_armor",
    32: "change_object_range",
    33: "change_object_speed",
    34: "heal_object",
    35: "teleport_object",
    36: "change_object_stance",
    37: "display_timer",
    38: "enable/disable_object",
    39: "enable/disable_technology",
    40: "change_object_cost",
    41: "set_player_visibility",
    42: "change_object_icon",
    43: "replace_object",
    44: "change_object_description",
    45: "change_player_name",
    46: "change_train_location",
    47: "change_research_location",
    48: "change_civilization_name",
    49: "create_garrisoned_object",
    50: "acknowledge_ai_signal",
    51: "modify_attribute",
    52: "modify_resource",
    53: "modify_resource_by_variable",
    56: "change_variable",
    57: "clear_timer",
})

empty_attributes = {
    "effect_type": -1,
    "ai_script_goal": -1,
    "aa_quantity": -1,
    "aa_armor_or_attack_type": -1,
    "quantity": -1,
    "tribute_list": -1,
    "diplomacy": -1,
    "number_of_units_selected": -1,
    "object_list_unit_id": -1,
    "player_source": -1,
    "player_target": -1,
    "technology": -1,
    "string_id": -1,
    "display_time": -1,
    "trigger_id": -1,
    "location_x": -1,
    "location_y": -1,
    "area_1_x": -1,
    "area_1_y": -1,
    "area_2_x": -1,
    "area_2_y": -1,
    "object_group": -1,
    "object_type": -1,
    "instruction_panel_position": -1,
    "attack_stance": -1,
    "time_unit": -1,
    "enabled_or_victory": -1,
    "food": -1,
    "wood": -1,
    "stone": -1,
    "gold": -1,
    "item_id": -1,
    "flash_object": -1,
    "force_research_technology": -1,
    "visibility_state": -1,
    "scroll": -1,
    "operation": -1,
    "object_list_unit_id_2": -1,
    "button_location": -1,
    "ai_signal_value": -1,
    "object_attributes": -1,
    "from_variable": -1,
    "variable_or_timer": -1,
    "facet": -1,
    "play_sound": -1,
    "message": '',
    "sound_name": '',
    "selected_object_id": -1,
}

default_attributes = {
    0: {
        "effect_type": 0,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    50: {
        "effect_type": 50,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": 0,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    8: {
        "effect_type": 8,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    10: {
        "effect_type": 10,
        "ai_script_goal": 1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    30: {
        "effect_type": 30,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    48: {
        "effect_type": 48,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": " ",
        "sound_name": "",
        "selected_object_id": [],
    },
    1: {
        "effect_type": 1,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": 0,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": 2,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    31: {
        "effect_type": 31,
        "ai_script_goal": -1,
        "aa_quantity": 1,
        "aa_armor_or_attack_type": 0,
        "quantity": [],
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": 3,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    28: {
        "effect_type": 28,
        "ai_script_goal": -1,
        "aa_quantity": 1,
        "aa_armor_or_attack_type": 0,
        "quantity": [],
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": 3,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    40: {
        "effect_type": 40,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    44: {
        "effect_type": 44,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": " ",
        "sound_name": "",
        "selected_object_id": [],
    },
    27: {
        "effect_type": 27,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": 2,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    42: {
        "effect_type": 42,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    26: {
        "effect_type": 26,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": " ",
        "sound_name": "",
        "selected_object_id": [],
    },
    32: {
        "effect_type": 32,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": 2,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    33: {
        "effect_type": 33,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    36: {
        "effect_type": 36,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": 0,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    18: {
        "effect_type": 18,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": 2,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": 0,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    45: {
        "effect_type": 45,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": " ",
        "sound_name": "",
        "selected_object_id": [],
    },
    47: {
        "effect_type": 47,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": 16,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": 82,
        "button_location": 7,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    46: {
        "effect_type": 46,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": 0,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    56: {
        "effect_type": 56,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": 1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": 0,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": " ",
        "sound_name": "",
        "selected_object_id": [],
    },
    16: {
        "effect_type": 16,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": 1,
        "operation": 1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": 0,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    21: {
        "effect_type": 21,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": 0,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    57: {
        "effect_type": 57,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": 0,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    49: {
        "effect_type": 49,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    11: {
        "effect_type": 11,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": 0,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    24: {
        "effect_type": 24,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    9: {
        "effect_type": 9,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    13: {
        "effect_type": 13,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": 1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    20: {
        "effect_type": 20,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": 10,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": 0,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": 0,
        "message": " ",
        "sound_name": " ",
        "selected_object_id": [],
    },
    37: {
        "effect_type": 37,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": 10,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": 0,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": 0,
        "facet": -1,
        "play_sound": -1,
        "message": " ",
        "sound_name": "",
        "selected_object_id": [],
    },
    38: {
        "effect_type": 38,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": 1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    39: {
        "effect_type": 39,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": 16,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": 1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    22: {
        "effect_type": 22,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    34: {
        "effect_type": 34,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    14: {
        "effect_type": 14,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    7: {
        "effect_type": 7,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    51: {
        "effect_type": 51,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": 1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": 0,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    52: {
        "effect_type": 52,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": 0,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": 1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    53: {
        "effect_type": 53,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": 0,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": 1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": 0,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    19: {
        "effect_type": 19,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    25: {
        "effect_type": 25,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    4: {
        "effect_type": 4,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": " ",
        "selected_object_id": [],
    },
    15: {
        "effect_type": 15,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    43: {
        "effect_type": 43,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": 2,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    2: {
        "effect_type": 2,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": 16,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": 0,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    3: {
        "effect_type": 3,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": " ",
        "sound_name": " ",
        "selected_object_id": [],
    },
    41: {
        "effect_type": 41,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": 2,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": 0,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    29: {
        "effect_type": 29,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    12: {
        "effect_type": 12,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    35: {
        "effect_type": 35,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    5: {
        "effect_type": 5,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": 0,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": 2,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    17: {
        "effect_type": 17,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": 1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    6: {
        "effect_type": 6,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    },
    23: {
        "effect_type": 23,
        "ai_script_goal": -1,
        "aa_quantity": [],
        "aa_armor_or_attack_type": [],
        "quantity": -1,
        "tribute_list": -1,
        "diplomacy": -1,
        "number_of_units_selected": -1,
        "object_list_unit_id": -1,
        "player_source": -1,
        "player_target": -1,
        "technology": -1,
        "string_id": -1,
        "display_time": -1,
        "trigger_id": -1,
        "location_x": -1,
        "location_y": -1,
        "area_1_x": -1,
        "area_1_y": -1,
        "area_2_x": -1,
        "area_2_y": -1,
        "object_group": -1,
        "object_type": -1,
        "instruction_panel_position": -1,
        "attack_stance": -1,
        "time_unit": -1,
        "enabled_or_victory": -1,
        "food": -1,
        "wood": -1,
        "stone": -1,
        "gold": -1,
        "item_id": -1,
        "flash_object": -1,
        "force_research_technology": -1,
        "visibility_state": -1,
        "scroll": -1,
        "operation": -1,
        "object_list_unit_id_2": -1,
        "button_location": -1,
        "ai_signal_value": -1,
        "object_attributes": -1,
        "from_variable": -1,
        "variable_or_timer": -1,
        "facet": -1,
        "play_sound": -1,
        "message": "",
        "sound_name": "",
        "selected_object_id": [],
    }
}

attributes = {
    0: [
        "effect_type",
    ],
    50: [
        "effect_type",
        "ai_signal_value",
    ],
    8: [
        "effect_type",
        "trigger_id",
    ],
    10: [
        "effect_type",
        "ai_script_goal",
    ],
    30: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "location_x",
        "location_y",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    48: [
        "effect_type",
        "player_source",
        "string_id",
        "message",
    ],
    1: [
        "effect_type",
        "diplomacy",
        "player_source",
        "player_target",
    ],
    31: [
        "effect_type",
        "aa_quantity",
        "aa_armor_or_attack_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "operation",
        "selected_object_id",
    ],
    28: [
        "effect_type",
        "aa_quantity",
        "aa_armor_or_attack_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "operation",
        "selected_object_id",
    ],
    40: [
        "effect_type",
        "object_list_unit_id",
        "player_source",
        "food",
        "wood",
        "stone",
        "gold",
    ],
    44: [
        "effect_type",
        "object_list_unit_id",
        "player_source",
        "string_id",
        "message",
    ],
    27: [
        "effect_type",
        "quantity",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "operation",
        "selected_object_id",
    ],
    42: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "object_list_unit_id_2",
        "selected_object_id",
    ],
    26: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "string_id",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "message",
        "selected_object_id",
    ],
    32: [
        "effect_type",
        "quantity",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "operation",
        "selected_object_id",
    ],
    33: [
        "effect_type",
        "quantity",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    36: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "attack_stance",
        "selected_object_id",
    ],
    18: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "player_target",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "flash_object",
        "selected_object_id",
    ],
    45: [
        "effect_type",
        "player_source",
        "string_id",
        "message",
    ],
    47: [
        "effect_type",
        "player_source",
        "technology",
        "object_list_unit_id_2",
        "button_location",
    ],
    46: [
        "effect_type",
        "object_list_unit_id",
        "player_source",
        "object_list_unit_id_2",
        "button_location",
    ],
    56: [
        "effect_type",
        "quantity",
        "operation",
        "from_variable",
        "message",
    ],
    16: [
        "effect_type",
        "player_source",
        "location_x",
        "location_y",
        "scroll",
    ],
    21: [
        "effect_type",
        "instruction_panel_position",
    ],
    57: [
        "effect_type",
        "variable_or_timer",
    ],
    49: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_list_unit_id_2",
        "selected_object_id",
    ],
    11: [
        "effect_type",
        "object_list_unit_id",
        "player_source",
        "location_x",
        "location_y",
        "item_id",
        "facet",
    ],
    24: [
        "effect_type",
        "quantity",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    9: [
        "effect_type",
        "trigger_id",
    ],
    13: [
        "effect_type",
        "player_source",
        "enabled_or_victory",
    ],
    20: [
        "effect_type",
        "object_list_unit_id",
        "player_source",
        "string_id",
        "display_time",
        "instruction_panel_position",
        "play_sound",
        "message",
        "sound_name",
    ],
    37: [
        "effect_type",
        "string_id",
        "display_time",
        "time_unit",
        "variable_or_timer",
        "message",
    ],
    38: [
        "effect_type",
        "object_list_unit_id",
        "player_source",
        "enabled_or_victory",
        "item_id",
    ],
    39: [
        "effect_type",
        "player_source",
        "technology",
        "enabled_or_victory",
        "item_id",
    ],
    22: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    34: [
        "effect_type",
        "quantity",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    14: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    7: [
        "effect_type",
        "number_of_units_selected",
        "selected_object_id",
    ],
    51: [
        "effect_type",
        "quantity",
        "object_list_unit_id",
        "player_source",
        "item_id",
        "operation",
        "object_attributes",
    ],
    52: [
        "effect_type",
        "quantity",
        "tribute_list",
        "player_source",
        "item_id",
        "operation",
    ],
    53: [
        "effect_type",
        "tribute_list",
        "player_source",
        "item_id",
        "operation",
        "from_variable",
    ],
    19: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "location_x",
        "location_y",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    25: [
        "effect_type",
        "object_list_unit_id",
        "player_source",
        "location_x",
        "location_y",
    ],
    4: [
        "effect_type",
        "player_source",
        "location_x",
        "location_y",
        "sound_name",
    ],
    15: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    43: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "player_target",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "object_list_unit_id_2",
        "selected_object_id",
    ],
    2: [
        "effect_type",
        "player_source",
        "technology",
        "force_research_technology",
    ],
    3: [
        "effect_type",
        "player_source",
        "string_id",
        "message",
        "sound_name",
    ],
    41: [
        "effect_type",
        "player_source",
        "player_target",
        "visibility_state",
    ],
    29: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    12: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "location_x",
        "location_y",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    35: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "location_x",
        "location_y",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    5: [
        "effect_type",
        "quantity",
        "tribute_list",
        "player_source",
        "player_target",
    ],
    17: [
        "effect_type",
        "number_of_units_selected",
        "object_list_unit_id",
        "player_source",
        "location_x",
        "location_y",
        "area_1_x",
        "area_1_y",
        "area_2_x",
        "area_2_y",
        "object_group",
        "object_type",
        "selected_object_id",
    ],
    6: [
        "effect_type",
        "number_of_units_selected",
        "selected_object_id",
    ],
    23: [
        "effect_type",
    ]
}

# for effect_id in range(0, 58):
#     try:
#         params = parameters[effect_id]
#         print("\"\"\"Attributes for the **xxxxx** effect are: \\n", end="")
#         for parameter in params:
#             if parameter is not "effect_type":
#                 print("\n-", parameter, end="")
#     except KeyError:
#         continue
#     print("\"\"\"")



# File & Folder setup
from AoE2ScenarioParser.aoe2_scenario import AoE2Scenario
from AoE2ScenarioParser.datasets.conditions import Condition
from AoE2ScenarioParser.datasets.effects import Effect
from AoE2ScenarioParser.datasets.players import Player
from AoE2ScenarioParser.datasets.trigger_lists import DiplomacyState, Operator, ButtonLocation, PanelLocation, \
    TimeUnit, VisibilityState, DifficultyLevel, TechnologyState, Comparison, ObjectAttribute, Attribute

scenario_folder = "C:/Users/Kerwin Sneijders/Games/Age of Empires 2 DE/76561198140740017/resources/_common/scenario/"
read_file = scenario_folder + "AllEffectConditionLists.aoe2scenario"
write_to_file = scenario_folder + "AllEffectConditionLists2.aoe2scenario"

scenario = AoE2Scenario(read_file)

trigger_manager = scenario.object_manager.trigger_manager

trigger = trigger_manager.add_trigger("Inform Betrayal!")
condition = trigger.add_condition(Condition.DIPLOMACY_STATE)
condition.amount_or_quantity = DiplomacyState.ALLY
condition.player = Player.TWO
condition.target_player = Player.THREE

effect = trigger.add_effect(Effect.DISPLAY_INSTRUCTIONS)
effect.player_source = Player.ONE
effect.message = "Spy: Your ally has betrayed you! He allied the enemy!"
effect.instruction_panel_position = PanelLocation.CENTER
effect.display_time = 10


scenario.write_to_file(write_to_file)






from AoE2ScenarioParser.pieces import aoe2_piece
from AoE2ScenarioParser.helper.retriever import Retriever
from AoE2ScenarioParser.helper.datatype import DataType


class CinematicsPiece(aoe2_piece.AoE2Piece):
    def __init__(self, parser_obj=None, data=None):
        retrievers = [
            Retriever("ASCII, Pregame cinematic filename", DataType("str16")),
            Retriever("ASCII, Victory cinematic filename", DataType("str16")),
            Retriever("ASCII, Loss cinematic filename", DataType("str16")),
            # Retriever("Separator (! in some version)", DataType("1")),
        ]

        super().__init__("Cinematics", retrievers, parser_obj, data=data)



from __future__ import annotations

import math

from AoE2ScenarioParser.datasets import units, buildings
from AoE2ScenarioParser.datasets.players import Player
from AoE2ScenarioParser.helper import helper
from AoE2ScenarioParser.helper.helper import Tile
from AoE2ScenarioParser.helper.retriever import find_retriever
from AoE2ScenarioParser.objects.aoe2_object import AoE2Object
from AoE2ScenarioParser.pieces.structs.unit import UnitStruct


class UnitObject(AoE2Object):
    def __init__(self,
                 player: Player,
                 x: float,
                 y: float,
                 z: float,
                 reference_id: int,
                 unit_const: int,
                 status: int,
                 rotation: float,
                 animation_frame: int,
                 garrisoned_in_id: int
                 ):

        self._player: Player = player
        """
        PLEASE NOTE: This is an internal (read-only) value for ease of access. It accurately represent the actual 
        player controlling the unit but is not directly connected to it. Changing this value will have no impact to your
        scenario.
        To change which player controls this unit, use:
            unit_manager.change_ownership(UnitObject, to_player[, from_player, skip_gaia])
        """
        self.x: float = x
        self.y: float = y
        self.z: float = z
        self.reference_id: int = reference_id
        self.unit_id: int = unit_const
        self.status: int = status
        self.rotation: float = rotation % math.tau
        # Mods by tau because the scenario editor seems to place units at radian angles not strictly less than tau.
        self.animation_frame: int = animation_frame
        self.garrisoned_in_id: int = garrisoned_in_id

        super().__init__()

    @property
    def player(self) -> Player:
        """
        PLEASE NOTE: This is an internal (read-only) value for ease of access. It DOES accurately represent the actual
        player controlling the unit BUT IT IS NOT directly connected to it. Changing this value will have no impact to
        your scenario.
        To change which player controls this unit, use:
            unit_manager.change_ownership(UnitObject, to_player[, from_player, skip_gaia])
        """
        return self._player

    @property
    def rotation(self) -> float:
        return self._rotation

    @rotation.setter
    def rotation(self, rotation: float) -> None:
        """Rotation in radians"""
        if not 0.0 <= rotation <= math.tau:
            raise ValueError(f'The Rotation value must be between 0 and tau (excl).')
        self._rotation = rotation

    @property
    def tile(self) -> Tile:
        return Tile(math.floor(self.x), math.floor(self.y))
        # Floor x and y as location (0.9, 0.9) is still Tile[x=0, y=0]

    @tile.setter
    def tile(self, tile: Tile) -> None:
        self.x = tile.x
        self.y = tile.y

    @property
    def name(self) -> str:
        try:
            return helper.pretty_print_name(units.unit_names[self.unit_id])
        except KeyError:  # Object wasn't a unit, maybe a building?
            return helper.pretty_print_name(buildings.building_names[self.unit_id])

    @staticmethod
    def _parse_object(parsed_data, **kwargs) -> UnitObject:  # Expected {unit=unitStruct, player=Player}
        unit = kwargs['unit']

        return UnitObject(
            player=kwargs['player'],
            x=find_retriever(unit.retrievers, "X position").data,
            y=find_retriever(unit.retrievers, "Y position").data,
            z=find_retriever(unit.retrievers, "Z position").data,
            reference_id=find_retriever(unit.retrievers, "ID").data,
            unit_const=find_retriever(unit.retrievers, "Unit 'constant'").data,
            status=find_retriever(unit.retrievers, "Status").data,
            rotation=find_retriever(unit.retrievers, "Rotation, in radians").data,
            animation_frame=find_retriever(unit.retrievers, "Initial animation frame").data,
            garrisoned_in_id=find_retriever(unit.retrievers, "Garrisoned in: ID").data,
        )

    @staticmethod
    def _reconstruct_object(parsed_header, parsed_data, objects, **kwargs) -> None:  # Expected {unit=unit_obj, units=units_list}
        unit_obj = kwargs['unit']
        units_list = kwargs['units']

        data_list = [value for key, value in vars(unit_obj).items()]
        del data_list[0]  # Remove player attribute

        units_list.append(UnitStruct(data=data_list))



from AoE2ScenarioParser.objects.aoe2_object import AoE2Object


class DiplomacyObject(AoE2Object):
    def __init__(self,
                 player_stances
                 ):

        self.player_stances = player_stances

        super().__init__()

    @staticmethod
    def _parse_object(parsed_data, **kwargs):
        pass

    @staticmethod
    def _reconstruct_object(parsed_header, parsed_data, objects, **kwargs):
        pass



from AoE2ScenarioParser.pieces import aoe2_piece
from AoE2ScenarioParser.helper.datatype import DataType
from AoE2ScenarioParser.helper.retriever import Retriever
from AoE2ScenarioParser.pieces.structs.player_data_one import PlayerDataOneStruct


class DataHeaderPiece(aoe2_piece.AoE2Piece):
    def __init__(self, parser_obj=None, data=None):
        retrievers = [
            Retriever("Next unit ID to place", DataType("u32")),
            Retriever("Version", DataType("f32")),
            Retriever("Player names", DataType("c256", repeat=16)),
            Retriever("String table player names", DataType("u32", repeat=16)),
            Retriever("Player data#1", DataType(PlayerDataOneStruct, repeat=16)),
            Retriever("Conquest mode", DataType("u8")),
            Retriever("Mission items Counter", DataType("u16"), save_as="mic"),
            Retriever("Mission available", DataType("u16")),
            Retriever("Mission timeline", DataType("f32")),
            Retriever("Mission item", DataType("30"), set_repeat="{mic}"),
            Retriever("Unknown", DataType("64")),
            Retriever("Filename", DataType("str16"))
        ]

        super().__init__("Data Header", retrievers, parser_obj, data=data)
